<?php
/**
 * Core Design Attachments plugin for Joomla! 1.5
 * @author		Daniel Rataj, <info@greatjoomla.com>
 * @package		Joomla
 * @subpackage	Content
 * @category	Plugin
 * @version		1.0.5
 * @copyright	Copyright (C) 2007 - 2010 Great Joomla!, http://www.greatjoomla.com
 * @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL 3
 * 
 * This file is part of Great Joomla! extension.   
 * This extension is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This extension is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

// Import library dependencies
jimport('joomla.plugin.plugin');
jimport('joomla.filesystem.file');
jimport('joomla.filesystem.path');
jimport('joomla.filesystem.folder');

/**
 * Core Design Attachments plugin
 *
 * @author		Daniel Rataj <info@greatjoomla.com>
 * @package		Core Design
 * @subpackage	System
 */
class plgContentCdAttachments extends JPlugin
{
	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @access	protected
	 * @param	object		$subject The object to observe
	 * @since	1.0
	 */
	function plgContentCdAttachments(&$subject)
	{
		parent::__construct($subject);
		
		// set extension name
		$this->plgname = 'cdattachments';
		
		// load plugin parameters
		$this->plugin = &JPluginHelper::getPlugin('content', $this->plgname);
		$this->params = new JParameter($this->plugin->params);

		// load language
		JPlugin::loadLanguage('plg_content_' . $this->plgname, JPATH_ADMINISTRATOR);
	}

	/**
	 * Joomla! onAfterDispatch() function
	 * 
	 * @return void
	 */
	function onAfterDispatch() {

		$task = JRequest::getCmd('attachments', '', 'post');

		switch ($task) {
			case 'getItems':
				$this->close($this->previewTemplate($this->getItems()));
				break;
			case 'download':
				$this->download();
				break;
			case 'fileTemplate':
				$this->close($this->fileTemplate());
				break;
			case 'addAttachment':
				$this->addAttachment();
				break;
			case 'editAttachment':
				$this->editAttachment();
				break;
			case 'deleteAttachment':
				$this->close($this->deleteAttachment());
				break;
			case 'filemgmt':
				$this->close($this->filemgmt());
				break;
			case 'uploadFile':
				$this->uploadFile();
				break;
			case 'removeFile':
				$this->close($this->removeFile());
				break;
			case 'checkPassword':
				$this->close($this->checkPassword());
				break;
			default:
				return;
				break;
		}
		
	}

	/**
	 * Joomla! onPrepareContent() function
	 * 
	 * @return string
	 */
	function onPrepareContent(&$article, &$params, $limitstart=0)
	{
		
		$document = &JFactory::getDocument(); // set document for next usage

		if ($document->getType() != 'html') return false; // prevent script loading in PDF or RSS output
		
		// disable on frontpage
		if ($this->params->get('disable_frontpage', 0) and JRequest::getCmd('option') == 'com_content' and JRequest::getCmd('view') == 'frontpage') return false;
		
		// disable category and section view
		$layout = JRequest::getCmd('layout');
		if (JRequest::getCmd('view', '') === 'section') {
			if ($layout == 'blog') {
				if ($this->params->get('disable_sectionview', 0)) return false;
			} else {
				return false;
			}
		}
		if (JRequest::getCmd('view', '') === 'category') {
			if ($layout == 'blog') {
				if ($this->params->get('disable_categoryview', 0)) return false;
			} else {
				return false;
			}
		}
		
		// disable section, category or article if required
		jimport('joomla.utilities.arrayhelper');

		$disablesection = explode(',', trim($this->params->get('disablesection' , '')));
		JArrayHelper::toInteger($disablesection);

		$disablecategory = explode(',', trim($this->params->get('disablecategory' , '')));
		JArrayHelper::toInteger($disablecategory);
		
		$disablearticle = explode(',', trim($this->params->get('disablearticle' , '')));
		JArrayHelper::toInteger($disablearticle);
		
		if (isset($article->sectionid) and in_array($article->sectionid, $disablesection)) return false;
		if (isset($article->catid) and in_array($article->catid, $disablecategory)) return false;
		if (isset($article->id) and in_array($article->id, $disablearticle)) return false;
		
		// Scriptegrator check
		if (!class_exists('JScriptegrator')) {
			JError::raiseNotice('', JText::_('CDATTACHMENTS_ENABLE_SCRIPTEGRATOR'));
			return false;
		} else {
			$message = JScriptegrator::check('1.5.1', 'jquery', 'site');
			if ($message)  {
			   JError::raiseNotice('', $message);
			   return false;
      		}
		}
		
		// check attachment directory
		$checkDir = $this->checkDir();
		if ($checkDir) {
			JError::raiseNotice('', JText::_($checkDir));
			return false;
		}
		
		// add JTable
		JTable::addIncludePath(array(
			dirname(__FILE__) . DS . $this->plgname . DS . 'utils' . DS . 'php'. DS . 'table')
		);
		
		// check if db table exists
		$row = $this->dbInstance();
		if (!$row->createDb()) {
			JError::raiseNotice('', $row->getError());
			return false;
		}
	  
		$live_path = JURI::root(true) . '/'; // define live site

		$uitheme = $this->params->get('uitheme', 'ui-lightness');
		$attachmentsCount = $row->attachmentsCount($article->id);
		
		$regex = '#{attachments}#is';
		
		$appendTheme = false;
		
		$button_attachments = '';
		$layoutpath = $this->getLayoutPath('button');
		
		ob_start();
			require($layoutpath);
			$button_attachments .= ob_get_contents();
		ob_end_clean();
		
		// button in article if attachments 
		if ($attachmentsCount) {
			if (preg_match($regex, $article->text)) {
				// append to article somewhere
				$article->text = preg_replace($regex, $button_attachments, $article->text);
			} else {
				// append at end
				$article->text .= $button_attachments;
			}
			$appendTheme = true;
		} else {
			// logged user has a permission to see the attachment button
			if ($this->hasEditPermission()) {
				// append to article somewhere
				if (preg_match($regex, $article->text)) {
					$article->text = preg_replace($regex, $button_attachments, $article->text);
				} else {
					// append at end
					$article->text .= $button_attachments;
				}
				$appendTheme = true;
			} else {
				if (preg_match($regex, $article->text)) {
					$article->text = preg_replace($regex, '', $article->text);
				} else {
					// append at end
					$article->text .= '';
				}
			}
		}
		
		static $once = false;
		if (!$once and $appendTheme) {

			$template = $this->params->get('template', 'default');
			
			// add CSS stylesheet and JavaScript file
			$document->addStyleSheet($live_path . 'plugins/content/' . $this->plgname . '/tmpl/' . $template . '/css/' . $this->plgname . '.css', 'text/css' );
			// add CSS stylesheet for RTL languages
			if ($document->direction === 'rtl') {
				$document->addStyleSheet($live_path . 'plugins/content/' . $this->plgname . '/tmpl/' . $template . '/css/' . $this->plgname . '_rtl.css', 'text/css');
			}
			
			$document->addScript($live_path . 'plugins/content/' . $this->plgname . '/tmpl/' . $template . '/js/jquery.' . $this->plgname . '.js');
			$document->addScript($live_path . 'plugins/content/' . $this->plgname . '/utils/js/jquery.upload-1.0.2.js');
				
			JScriptegrator::importUI();
			JScriptegrator::importUITheme($uitheme);
			
			$token = $this->getToken();
			
			$document->addScriptDeclaration("
			<!--
			if (typeof(jQuery) === 'function') {
				jQuery(document).ready(function($){
					if (typeof($.fn.$this->plgname) === 'function') {
						$('div.attachments button').cdattachments({
							settings: {
								uitheme : '$uitheme',
								token : '$token',
								count : " . str_replace(array('(', ')'), '', $attachmentsCount) . ",
								addnewauth : " . ($this->hasEditPermission() ? 1 : 0) . ",
								filemanagerauth : " . ($this->filemanagerPermission() ? 1 : 0) . "
		    				},
		    				language: {
		    					CDATTACHMENTS_ATTACHMENTS : '" . JText::_('CDATTACHMENTS_ATTACHMENTS', true) . "',
		    					CDATTACHMENTS_CLOSE : '" . JText::_('CDATTACHMENTS_CLOSE', true) . "',
		    					CDATTACHMENTS_ADDNEW : '" . JText::_('CDATTACHMENTS_ADDNEW', true) . "',
		    					CDATTACHMENTS_BACK : '" . JText::_('CDATTACHMENTS_BACK', true) . "',
		    					CDATTACHMENTS_EMPTY_VALUE : '" . JText::_('CDATTACHMENTS_EMPTY_VALUE', true) . "',
		    					CDATTACHMENTS_FILE_IS_PROTECTED : '" . JText::_('CDATTACHMENTS_FILE_IS_PROTECTED', true) . "',
		    					CDATTACHMENTS_FILEINFORMATION : '" . JText::_('CDATTACHMENTS_FILEINFORMATION', true) . "',
		    					CDATTACHMENTS_REALLY : '" . JText::_('CDATTACHMENTS_REALLY', true) . "',
		    					CDATTACHMENTS_SAVED : '" . JText::_('CDATTACHMENTS_SAVED', true) . "',
		    					CDATTACHMENTS_EDITED : '" . JText::_('CDATTACHMENTS_EDITED', true) . "',
		    					CDATTACHMENTS_DELETED : '" . JText::_('CDATTACHMENTS_DELETED', true) . "',
		    					CDATTACHMENTS_FILES : '" . JText::_('CDATTACHMENTS_FILES', true) . "',
		    					CDATTACHMENTS_SELECT_FILE : '" . JText::_('CDATTACHMENTS_SELECT_FILE', true) . "',
		    					CDATTACHMENTS_PASSWORD_INCORRECT : '" . JText::_('CDATTACHMENTS_PASSWORD_INCORRECT', true) . "',
		    					CDATTACHMENTS_NO_ATTACHMENTS : '" . JText::_('CDATTACHMENTS_NO_ATTACHMENTS', true) . "'
		    				}
		    			});
					}
				});
			}
			// -->");
			
			$once = true;
		}
		
		return $article->text;

	}

	/**
	 * Method to determine a hash for anti-spoofing variable names
	 *
	 * @return	string	Hashed var name
	 */
	function getToken() {
		jimport('joomla.utilities.utility');
		$token = JUtility::getToken();
		return $token;
	}

	/**
	 * Check attachment directory
	 *
	 * Check directory routine
	 *
	 * @return string
	 */
	function checkDir() {

		$dir = $this->attachmentsDir();

		// check if dir exists
		if (!JFolder::exists($dir)) {
			return JText::_('CDATTACHMENTS_CHECK_DIR');
		}

		// write .htaccess file
		if (!JFile::write($dir . DS . '.htaccess', 'deny from all')) {
			return JText::_('CDATTACHMENTS_DIR_WRITEABLE');
		}

		// write blank index.htnl
		if (!JFile::write($dir . DS . 'index.html', '<html><body bgcolor="#FFFFFF"></body></html>')) {
			return JText::_('CDATTACHMENTS_DIR_WRITEABLE');
		}

		return false;

	}

	/**
	 * Global close wrapper
	 *
	 * @param $msg		message to display
	 *
	 * @return void
	 */
	function close($msg = '') {
		global $mainframe;
		$mainframe->close($msg);
	}

	/**
	 * Load attachments
	 *
	 * Load attachments from database
	 *
	 * @return object
	 */
	function getItems() {
		
		$row = $this->dbInstance();
		
		$orderby = $this->params->get('orderby', 'modified');
		$orderdir = $this->params->get('orderdir', 'ASC');
		
		return $row->getItems($orderby, $orderdir);
	}

	/**
	 * Run template
	 *
	 * Put database object list to the template.�
	 *
	 * @param	$object		file information
	 * @return	string
	 */
	function previewTemplate($object) {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );
		
		$template = $this->params->get('template', 'default');
		
		$count = count($object);
		
		$layoutpath = $this->getLayoutPath('view');
		
		$tmpl = '';
		
		ob_start();
		require_once($layoutpath);
		$tmpl .= ob_get_contents();
		ob_end_clean();
		
		return $tmpl;
	}

	/**
	 * Get username via document owner
	 *
	 * @param $userid	user ID
	 *
	 * @return string	username
	 */
	function getUsername($userid = 0) {
		$user =& JFactory::getUser($userid);
		return $user->username;
	}

	/**
	 * Template to add new item
	 *
	 * @return string
	 */

	function fileTemplate() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		$fileid = JRequest::getInt('fileid', 0, 'post');
		
		if (!$this->hasEditPermission($fileid)) return JText::_('CDATTACHMENTS_NOT_ALLOWED');
		
		if(!JFolder::files($this->attachmentsDir(), '.', false, false, array('.htaccess', 'index.html', '.svn', 'CVS'))) return JText::_('CDATTACHMENTS_NO_FILES');
		
		$mode = ($fileid ? 'editAttachment' : 'addAttachment');

		$fileinfo = $this->dbFileInfo($fileid);
		
		$files = JFolder::files($this->attachmentsDir(), '.', false, false, array('.htaccess', 'index.html'));

		$tmpl = '';
		
		ob_start();
		require_once($this->getLayoutPath('form'));
		$tmpl .= ob_get_contents();
		ob_end_clean();
		
		return $tmpl;
	}

	/**
	 *
	 * @return string	Info icon image
	 */
	function infoIcon($title = '') {
		return '<img src="' . JURI::base(true) . '/plugins/content/' . $this->plgname . '/images/icon_info.png" title="' . $title . '" alt="" style="margin-left: 5px; width: 16px; height: 16px;" width="16" height="16" />';
	}

	/**
	 * Save file
	 *
	 * @return void
	 */

	function addAttachment() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );
		
		if (!$this->hasEditPermission()) $this->close(JText::_('CDATTACHMENTS_NOT_ALLOWED'));
		
		$data ['file'] = JRequest::getString('file', '', 'post');
		$data ['size'] = filesize($this->filePath($data ['file']));
		$data ['type'] = $this->mimetype($data ['file']);
		$data ['title'] = JRequest::getString('filetitle', '', 'post');
		$data ['desc'] = JRequest::getString('filedesc', '', 'post', JREQUEST_ALLOWHTML);
		$data ['password'] = JRequest::getString('filepassword', '', 'post');
		$data ['password'] = ($data ['password'] ? md5($data ['password']) : '');
		$data ['access'] = JRequest::getInt('filelevel', 0, 'post');
		$data ['articleid'] = JRequest::getInt('articleid', 0, 'post');
		
		$now =& JFactory::getDate();
		$data ['created'] = $now->toMySQL();
		
		$user	=& JFactory::getUser();
		$data ['userid'] = $user->id;
		
		$data ['published'] = 1;
		
		$row = $this->dbInstance();
		if (!$row->bind($data)) $this->close($this->getError());
		if (!$row->check()) $this->close($this->getError());
		if (!$row->store()) $this->close($this->getError());

		$this->close();
	}

	/**
	 * Edit file
	 *
	 * @return void
	 */

	function editAttachment() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );
		
		global $mainframe;

		$fileid = JRequest::getInt('fileid', 0, 'post');
		if (!$this->hasEditPermission($fileid)) $this->close(JText::_('CDATTACHMENTS_NOT_ALLOWED'));
		$fileinfo = $this->dbFileInfo($fileid);
		
		if ($fileid) $data['id'] = $fileid;
		
		if (JRequest::getString('file', '', 'post')) {
			$data['file'] = JRequest::getString('file', '', 'post');
		}
		
		// change size and type if necessary
		if ($fileinfo->file != $data['file']) {
			$data['size'] = filesize($this->attachmentsDir() . DS . $data['file']);
			$data['type'] = $this->mimetype($data['file']);
		}

		if (JRequest::getString('filetitle', stripslashes($fileinfo->title), 'post')) {
			$data['title'] = JRequest::getString('filetitle', stripslashes($fileinfo->title), 'post');
		}
		if (JRequest::getString('filetitle', stripslashes($fileinfo->title), 'post')) {
			$data['title'] = JRequest::getString('filetitle', stripslashes($fileinfo->title), 'post');
		}
		
		if (JRequest::getString('filedesc', stripslashes($fileinfo->desc), 'post', JREQUEST_ALLOWHTML)) {
			$data['desc'] = JRequest::getString('filedesc', stripslashes($fileinfo->desc), 'post', JREQUEST_ALLOWHTML);
		}
		
		$password = JRequest::getString('filepassword', '', 'post');
		
		if (isset($password)) {
			$data['password'] = JRequest::getString('filepassword', '', 'post');
			switch ($data['password']) {
				case '0':
					$data['password'] = '';
					break;
				default:
					$data['password'] = ($data['password'] ? md5($data['password']) : $fileinfo->password);
					break;
			}
		}
		
		$access = JRequest::getInt('filelevel', 0, 'post');
		if (isset($access)) {
			$data['access'] = $access;
		}
		
		$row = $this->dbInstance();
		if (!$row->bind($data)) $this->close($this->getError());
		if (!$row->check()) $this->close($this->getError());
		if (!$row->store()) $this->close($this->getError());

		$this->close();
	}

	/**
	 * File management
	 *
	 * @return string
	 */
	function filemgmt() {
		
		if (!$this->filemanagerPermission()) return JText::_('CDATTACHMENTS_NOT_ALLOWED');

		$files = JFolder::files($this->attachmentsDir(), '.', false, false, array('.svn', 'CVS', 'index.html', '.htaccess'));
		
		$template = $this->params->get('template', 'default');
		$layoutpath = $this->getLayoutPath('filemgmt');
		
		$tmpl = '';
		
		ob_start();
		require_once($layoutpath);
		$tmpl .= ob_get_contents();
		ob_end_clean();
		
		return $tmpl;
	}

	/**
	 * Upload file function
	 * @return void
	 */
	function uploadFile() {

		// Check for request forgeries - custom modification!
		if (JUtility::getToken() != JRequest::getVar( 'token', '', 'post', 'alnum' )) {
			jexit( 'Invalid Token' );
		}
		
		if (!$this->filemanagerPermission()) $this->close(JText::_('CDATTACHMENTS_NOPERMISSIONTOUPLOAD'));
		
		if (!isset($_FILES['file']) or $_FILES['file']['error']) {
			$this->close(JText::_('CDATTACHMENTS_UPLOAD_ERROR'));
		}

		$file_uploadname = $_FILES['file']['name'];
		$file_tmp_path = $_FILES['file']['tmp_name'];
		$file_mime = $_FILES['file']['type'];
		$file_size = $_FILES['file']['size'];
		

		// file not allowed to upload
		$allowedToUpload = $this->allowedToUpload($_FILES);
		if ($allowedToUpload) {
			$this->close($allowedToUpload);
		}

		// file already exists
		if (JFile::exists($this->filePath($file_uploadname))) {
			$this->close(JText::sprintf('CDATTACHMENTS_FILE_ALREADY_EXISTS', $file_uploadname));
		}

		if(JFile::upload($file_tmp_path, $this->filePath($file_uploadname))) {
			// file have been uploaded
			$this->close();
		} else {
			// uploading error
			$this->close(JText::_('CDATTACHMENTS_UPLOAD_ERROR'));
		}
	}

	/**
	 * Remove rfile
	 *
	 * @return string
	 */
	function removeFile() {
		
		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		if (!$this->filemanagerPermission()) $this->close(JText::_('CDATTACHMENTS_NOPERMISSIONTODELETE'));
		
		$file = explode(',', JRequest::getString('file', '', 'post'));
		
		$msg = '';
		foreach ($file as $file_item) {
			if (!JFile::delete($this->attachmentsDir() . DS  . $file_item) and $file_item) {
				$msg .= JText::sprintf('CDATTACHMENTS_FILE_DELETE_ERROR', $file_item) . '<br />';
				break;
			}
		}

		$this->close($msg);

	}

	/**
	 * Delete file
	 *
	 * @return string	Empty if success, text if error.
	 */
	function deleteAttachment() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		$fileid = (int) JRequest::getInt('fileid', 0, 'post');

		$fileinfo = $this->dbFileInfo($fileid);

		if (!$this->hasEditPermission($fileid)) return JText::_('CDATTACHMENTS_NOPERMISSIONTODELETE');
		
		$row = $this->dbInstance();
		if (!$row->delete($fileid)) return $row->getError();
		return '';
	}

	/**
	 * Check if user is authorized to delete the file, file's author by default
	 *
	 * @param $articleid	int
	 *
	 * @return boolean
	 */
	function hasEditPermission($fileid = 0) {
		
		 $user =& JFactory::getUser();
		 $userid = (int) $user->id;
		 $gid = $user->gid;
	
		 if ($user->guest) return false; // disable for guests
		 if ($user->gid == '25') return true; // enable for superadministrator accounts
		 
		 $addnew_allowed = $this->params->get('addnew_restriction');
		 
		 // edit file if I'm file author
		 $file = $this->dbFileInfo($fileid);
		 if ($file) {
		 	if (is_array($addnew_allowed) and $userid == $file->userid and in_array($gid, $addnew_allowed)) return true;
		 	if (!is_array($addnew_allowed) and $userid == $file->userid and $addnew_allowed == $gid or $gid == '25') return true;
		 	return false;
		 }
		 if (is_array($addnew_allowed) and in_array($gid, $addnew_allowed)) return true;
		 if (!is_array($addnew_allowed) and $addnew_allowed == $gid or $gid == '25') return true;
	
		 return false;
	}
	
	/**
	 * Check if user has permission to upload the files
	 * 
	 * @return boolean
	 */
	function filemanagerPermission() {

		$user =& JFactory::getUser();
		$gid = $user->gid;
		if ($user->guest) return false; // disable for guests
		if ($gid == '25') return true; // enable for superadministrator accounts
		
		$filemanagement_allowed = $this->params->get('filemanager_restriction');
		
		// level array
		if (is_array($filemanagement_allowed) and in_array($gid, $filemanagement_allowed)) return true;
		
		// level string
		if (!is_array($filemanagement_allowed) and $filemanagement_allowed == $gid) return true;
		return false;
	}

	/**
	 * Check if filename is allowed to upload
	 *
	 * @param $file
	 *
	 * @return string	Error message. if empty, file is correct.
	 */
	function allowedToUpload($file = null) {

		if (!$file) return JText::_('CDATTACHMENTS_UPLOAD_ERROR');

		$file_uploadname = $_FILES['file']['name'];
		$file_mime = $_FILES['file']['type'];
		$file_size = $_FILES['file']['size'];

		if ($file_size > ((int)$this->params->get('reject_filesize' , 10) * 1000000)) {
			return JText::sprintf('CDATTACHMENTS_MAX_SIZE', (int)$this->params->get('reject_filesize' , 10));
		}

		$reject_filename = array_map('trim', explode("\n", trim($this->params->get('reject_filename' , ''))));
		array_push($reject_filename, '.htaccess', 'index.html', 'index.htm', 'index.php');

		$reject_ext = array_map('trim', explode("\n", trim($this->params->get('reject_ext' , ''))));
		if (in_array($file_uploadname, $reject_filename) or in_array(JFile::getExt($file_uploadname), $reject_ext)) {
			return JText::sprintf('CDATTACHMENTS_NOTALLOWED_TO_UPLOAD', $file_uploadname);
		}

		return '';
	}

	/**
	 * Atttachment directory
	 *
	 * Return absolute path to the attachment directory.
	 *
	 * @return string
	 */
	function attachmentsDir() {
		$abs_path = dirname(dirname(dirname(__FILE__)));
		$dir = JPath::clean($abs_path . DS . $this->params->get('directory', 'attachments'));

		return $dir;
	}

	/**
	 * Path to the file
	 *
	 * Return absolute path to the required file
	 *
	 * @param	$filename	filename
	 *
	 * @return string
	 */
	function filePath($filename = '') {
		if ($filename) {
			$path = JPath::clean($this->attachmentsDir() . DS . $filename);
		} else {
			$path = '';
		}
		return  $path;
	}

	/**
	 * Present a size (in bytes) as a human-readable value
	 *
	 * @param int    $bytes        size (in bytes)
	 *
	 * @return string
	 */
	function byte_convert($bytes = 0) {
		$symbol = array('B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB');

		$exp = 0;
		$converted_value = 0;
		if( $bytes > 0 )
		{
			$exp = floor( log($bytes)/log(1024) );
			$converted_value = ( $bytes/pow(1024,floor($exp)) );
		}

		return sprintf( '%.2f '.$symbol[$exp], $converted_value );
	}

	/**
	 * Get Joomla! levels
	 *
	 * @return array	Joomla! levels
	 */
	function levelList() {

		$db =& JFactory::getDBO();
		$query = 'SELECT id AS value, name AS text'
		. ' FROM #__groups'
		. ' ORDER BY id'
		;
		$db->setQuery( $query );
		$groups = $db->loadObjectList();
		
		$user =& JFactory::getUser();
		$aid = $user->aid;
		
		$groups_re = array();
		foreach($groups as $item) {
			if ($item->value <= $aid) {
				$groups_re []= $item;
			}
		}
		
		return $groups_re;

	}

	/**
	 * Find mime type
	 *
	 * @param $filename
	 * @return string	Mime type
	 */
	function mimetype($filename = '') {
		require_once (dirname(__file__) . DS . $this->plgname . DS . 'utils' . DS . 'php' . DS . 'mime_class.php');

		$mimetype = new cdAttachmentsHelperMimeType(); // load mime type
		$mimetype = $mimetype->getType(strtolower($filename)); // get mime type

		return $mimetype;
	}

	/**
	 * Download file
	 *
	 * @param $filepath		Path to the file.
	 *
	 * @return void
	 */
	function download() {

		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		global $mainframe;

		$fileid = JRequest::getInt('fileid', 0, 'post');

		$fileinfo = $this->dbFileInfo($fileid);
		
		if (!$fileinfo) {
			$mainframe->redirect($this->url(), JText::_('CDATTACHMENTS_ERROR'));
		}

		$filename = $fileinfo->file;

		$filepath = $this->filePath($filename);
		
		if (!JFile::exists($filepath)) $mainframe->redirect($this->url(), JText::sprintf('CDATTACHMENTS_FILE_NOT_EXISTS', $filename));

		// check if file is procted via password
		if ($fileinfo->password) {
			$filepassword = JRequest::getString('filepassword', '', 'post');
				
			if (!$filepassword or md5($filepassword) != $fileinfo->password) {
				$mainframe->redirect($this->url(), JText::_('CDATTACHMENTS_PASSWORD_INCORRECT'));
			}
		}

		// load required classes
		require_once (dirname(__file__) . DS . $this->plgname . DS . 'utils' . DS . 'php' . DS . 'download_class.php');

		$filename = basename($filepath); // filename
		$mimetype = $this->mimetype($filename); // get mime type

		$filepath = new DOWNLOADFILE($filepath, $mimetype, 'attachment', $filename); // load file to download

		if (!$filepath->df_download())
		{
			$mainframe->redirect($this->url(), JText::_('CDATTACHMENTS_ERROR'));
		} else
		{
			$row = $this->dbInstance();
			$row->hit($fileid);
			$this->close();
		}
	}

	/**
	 * Get actual URL
	 *
	 * @return string	URL
	 */
	function url() {
		jimport('joomla.enviroment.request');
		return JRequest::getURI();
	}

	/**
	 * Get file information from database
	 *
	 * @param $id
	 * @return object
	 */
	function dbFileInfo($id = 0) {
		$row = $this->dbInstance();
		if ($row->load($id)) return $row;
		return null;
	}
	
	/**
	 * Check password (ajax)
	 * 
	 * @return boolean		True, if password is correct.
	 */
	function checkPassword() {
		// Check for request forgeries
		JRequest::checkToken() or jexit( 'Invalid Token' );

		$fileid = JRequest::getInt('fileid', 0, 'post');
		
		// prevent empty file id
		if (!$fileid) return false;
		
		$fileinfo = $this->dbFileInfo($fileid);
		
		$filepassword = JRequest::getString('filepassword', '', 'post');
		
		// prevent empty password
		if (!$filepassword) return false;
		
		$filepassword = md5($filepassword);
		
		if ($fileinfo->password == $filepassword) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * Load DB instance
	 * 
	 * @return A database object
	 */
	function dbInstance() {
		jimport('joomla.database.table');
		$row = &JTable::getInstance('CdAttachments');
		return $row;
	}
	
	/**
	 * Layout path
	 * 
	 * @param $tmpl
	 * @return string		Path
	 */
	function getLayoutPath($layout = 'view') {
		$template = $this->params->get('template', 'default');
		return dirname(__FILE__) . DS . $this->plgname . DS . 'tmpl' . DS . $template . DS . $layout . '.php';
	}
}
?>