/**
 * Core Design Attachments plugin for Joomla! 1.5
 * @author		Daniel Rataj, <info@greatjoomla.com>
 * @subpackage	Content
 * @category	Plugin
 * @version		1.0.5
 * @copyright	Copyright (C) 2007 - 2010 Great Joomla!, http://www.greatjoomla.com
 * @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL 3
 * 
 * This file is part of Great Joomla! extension.   
 * This extension is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This extension is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

if (typeof(jQuery) == 'function') {
	
	(function($) {
		
		$.fn.cdattachments = function(options) {
			
			// set defaults
			var defaults = {
				language : {
					CDATTACHMENTS_ATTACHMENTS : "Attachments",
					CDATTACHMENTS_CLOSE : "Close",
					CDATTACHMENTS_ADDNEW : "Add new",
					CDATTACHMENTS_BACK : "Back",
					CDATTACHMENTS_EMPTY_VALUE : "Please fill in all required fields!",
					CDATTACHMENTS_FILE_IS_PROTECTED : "The file you requested is protected. Please enter the password.",
					CDATTACHMENTS_FILEINFORMATION : "File information",
					CDATTACHMENTS_REALLY : "Really?",
					CDATTACHMENTS_SAVED : "Attachment has been saved.",
					CDATTACHMENTS_EDITED : "Attachment has been updated.",
					CDATTACHMENTS_DELETED : "Attachment has been deleted.",
					CDATTACHMENTS_FILES : "Files",
					CDATTACHMENTS_SELECT_FILE : "Please select a file.",
					CDATTACHMENTS_PASSWORD_INCORRECT : "Password incorrect!",
					CDATTACHMENTS_NO_ATTACHMENTS : "No attachments"
				},
				settings : {
					uitheme : 'ui-lightness',
					token : 0,
					count : 0,
					addnewauth : 0,
					filemanagerauth : 0
				}
			},
			clickable = {},
			attachments = {},
			opts = $.extend(defaults, options);
			
			return this.each(function() {
				clickable = $(this);
				buttonsUI(clickable);
				
				// load attachments
				clickable.click(function() {
					var articleid_element = $(this).attr('id').split('_'); // global article id variable
					articleid = articleid_element[1] || 0;
					enableDialog();
				});
			});
			
			/**
			 * Enable dialog
			 * 
			 * @return void
			 */
			function enableDialog() {
				if ($('#attachments', 'body').length) return false;
				
				attachments = $('<div />', {
					id : 'attachments'
				});
				
				attachments.dialog({
					title: lang('ATTACHMENTS'),
					height: 400,
					width: 600,
					position: { at : 'center', my: 'center'},
					draggable: false,
					resizable: false,
					stack: false,
					buttons: {
						'addnew' : function() {
							addnew();
						},
						'files' : function() {
							filemgmt();
						},
						'back' : function() {
							load();
						},
						'close' : function() {
							$(this).dialog('close');
						}
					},
					open: function() {
						clickable.button('disable');
						theme($(this)); // add theme
						
						setButtonTitles(); // button routine
						buttonRoutine();
						
						load($(this)); // load attachments
						
					},
					close: function() {
						clickable.button('enable');
						removeDialog($(this));
					}
				});
			};
			
			/**
			 * Load attachments
			 * 
			 * @return void
			 */
			function load() {
				$.ajax({
					type: 'POST',				
					dataType: 'html',
					timeout: 8000,
					cache: true,
					data: 'attachments=getItems' + '&articleid=' + articleid + '&' + getParam('token') + '=1',
					beforeSend: function() {
						loading();
					},
					success: function(msg) {
						
						attachments.html(msg);
						buttonsUI($('button', attachments)); // add UI theme to buttons in container
						
						buttonRoutine();
						
						fileinformationicon(); // file info icon routine
						deleteIcon(); // delete icon routine
						editIcon(); // edit icon routine
						
						var form = $('form[name="downloadfile"]', attachments);
						form.submit(function(e) {
							
							form = $(this);
							
							// check password
							var password = $('input[name="filepassword"]:hidden', $(this));
							
							if (password.val() == '') {
								checkPassword(form);							
								e.preventDefault();
								return false;
							} else {
								return true;
							}
						});
					},
					error: function() {
						alert(lang('ERROR'));
						return false;
					},
					complete: function() {
						loading();
					}
				});
			};
			
			/**
			 * Ajax check password
			 * 
			 * @param form
			 * @return void
			 */
			function checkPassword(form) {
				
				var pass = $.trim(prompt(lang('FILE_IS_PROTECTED'), ''));
				
				if (pass != null && pass != '') {
					var fileid = $('input[name="fileid"]:hidden', form).val();
					
					var password = $('input[name="filepassword"]:hidden', form);
					
					$.ajax({
						type: 'POST',				
						dataType: 'html',
						timeout: 8000,
						cache: true,
						data: 'attachments=checkPassword' + '&fileid=' + fileid + '&filepassword=' + pass + '&' + getParam('token') + '=1',
						beforeSend: function() {
							loading();
						},
						success: function(msg) {
							if (msg) {
								password.val(pass);
								form.submit();
								return true;
							} else {
								alert(lang('PASSWORD_INCORRECT'));
								return false;
							}
						},
						error: function() {
							alert(lang('ERROR'));
							return false;
						},
						complete: function() {
							loading();
						}
					});
				}
			};
			
			/**
			 * File manager
			 * 
			 * @return void
			 */
			function filemgmt() {
				$.ajax({
					type: 'POST',				
					dataType: 'html',
					timeout: 8000,
					cache: true,
					data: 'attachments=filemgmt&' + getParam('token') + '=1',
					beforeSend: function() {
						loading();
					},
					success: function(msg) {
						buttonRoutine('filemgmt');
						attachments.html(msg);
						buttonsUI($('button', attachments)); // add UI theme to buttons in container
						
						// save upload button
						$('form[name="removefile"]', attachments).submit(function (e) {
							e.preventDefault();
							var select_file = $('select[name="file"]', $(this));
							if (select_file.val()) {
								$.ajax({
									type: 'POST',				
									dataType: 'html',
									timeout: 8000,
									cache: true,
									data: 'attachments=removeFile' + '&file=' + $('select[name="file"]', $(this)).val() + '&' + getParam('token') + '=1',
									beforeSend: function() {
										var prompt = confirm(lang('REALLY'));
										if (!prompt) return false;
										loading();
									},
									success: function(msg) {
										if (msg) {
											// error
											buttonRoutine('files');
											attachments.html(msg);
											return false;
										}
										// success
										$('select[name="file"]', $('form[name="removefile"]', attachments)).children('option:selected').remove();
									},
									error: function() {
										alert(lang('ERROR'));
										return false;
									},
									complete: function() {
										loading();
									}
								});
							} else {
								alert(lang('SELECT_FILE'));
								return false;
							}
						});
						// upload
						var file_input_clone = $('input:file', $('form[name="uploadfile"]', attachments));
						$('form[name="uploadfile"] button.file', attachments).click(function(e) {
							e.preventDefault();
							var file_input = $('input:file', $(this).parents('form'));
							if (!file_input.val()) return false;
							
							loading();
							file_input.upload(
								window.location.href,
								{ attachments : 'uploadFile', token : getParam('token') },
								function(res) {
									loading();
				                	if (typeof($(res).selector) === 'string' && $(res).selector) {
				                		alert($(res).selector);
				                		return false;
				                	}
				                	var file_input_value = basename(file_input.val());
				                	$('select[name="file"]', $('form[name="removefile"]', attachments)).append('<option value="' + file_input_value + '" title="' + file_input_value + '">' + file_input_value + '</option>');
				                	file_input.replaceWith('<input type="file" name="file" />');
				                	return true;
			            	}, 'text');
						});
					},
					error: function() {
						alert(lang('ERROR'));
						return false;
					},
					complete: function() {
						loading();
					}
				});
			};
			
			/**
			 * Add new
			 * 
			 * @return void
			 */
			function addnew() {
				$.ajax({
					type: 'POST',				
					dataType: 'html',
					timeout: 8000,
					cache: true,
					data: 'attachments=fileTemplate' + '&' + getParam('token') + '=1',
					beforeSend: function() {
						loading();
					},
					success: function(msg) {
						
						buttonRoutine('addnew');
						
						attachments.html(msg);
						buttonsUI($('button', attachments));
						
						var form = $('form[name="fileform"]', attachments);
						
						$('input[name="filetitle"]', form).focus();
						
						form.submit(function(e) {
							
							e.preventDefault();
							
							var emptylang = lang('EMPTY_VALUE');
							
							var filetitle = $('input[name="filetitle"]', $(this));
							if (filetitle.val() == '') {
								alert(emptylang);
								filetitle.focus();
								return false;
							}
							
							var file = $('select[name="file"]', $(this));
							if (file.val() == '') {
								alert(emptylang);
								file.focus();
								return false;
							}
							
							$('input[name="articleid"]:hidden', $(this)).val(articleid); // insert article id to hidden input
							
							$.ajax({
								type: 'POST',	
								dataType: 'html',
								timeout: 8000,
								cache: true,
								data: $(this).serialize(),
								beforeSend: function() {
									loading();
								},
								success: function(msg) {
									if (msg) {
										alert(msg);
										return false;
									} else {
										// re-count
										var count = $('button[id="cdattachments_' + articleid + '"]').button('option', 'label').match(/\((\d*?)\)/),
										label = $('button[id="cdattachments_' + articleid + '"]').button('option', 'label');
										$('button[id="cdattachments_' + articleid + '"]').button('option', 'label', label.replace(/\(\d*?\)/i, '(' + ((count[1] * 1) + 1) + ')'));
										
										successInfo(lang('SAVED'));
									}
								},
								error: function() {
									alert(lang('ERROR'));
									return false;
								},
								complete: function() {
									loading();
								}
							});
							
						});
						
					},
					error: function() {
						alert(lang('ERROR'));
						return false;
					},
					complete: function() {
						loading();
					}
				});
			};
			
			/**
			 * Activate delete icon routine
			 * 
			 * @return void
			 */
			function deleteIcon() {
				$('div.document div.fileicons div.fileicondelete', attachments).click(function() {
					
					var really = confirm(lang('REALLY'));
					if (!really) return false;
					
					var document = $(this).parent('div.fileicons').parent('div.document');
					var fileid = $('input[name="fileid"]:hidden', $('div.downloadbutton', document)).val();
					
					$.ajax({
						type: 'POST',				
						dataType: 'html',
						timeout: 8000,
						cache: true,
						data: 'attachments=deleteAttachment' + '&fileid=' + fileid + '&' + getParam('token') + '=1',
						beforeSend: function() {
							loading();
						},
						success: function(msg) {
							if (msg) {
								alert(msg);
								return false;
							} else {
								document.slideUp('slow', function() {
									$(this).remove();
									if ($('div.document', attachments).length < 1) {
										attachments.html(lang('NO_ATTACHMENTS'));
									}
									
									// re-count
									var count = $('button[id="cdattachments_' + articleid + '"]').button('option', 'label').match(/\((\d*?)\)/),
									label = $('button[id="cdattachments_' + articleid + '"]').button('option', 'label');
									$('button[id="cdattachments_' + articleid + '"]').button('option', 'label', label.replace(/\(\d*?\)/i, '(' + ((count[1] * 1) - 1) + ')'));
								});
								
							}
						},
						error: function() {
							alert(lang('ERROR'));
							return false;
						},
						complete: function() {
							loading();
						}
					});
					
				});
			};
			
			/**
			 * Activate edit icon function
			 * 
			 * @return void
			 */
			function editIcon() {
				
				$('div.document div.fileicons div.fileiconedit', attachments).click(function() {
					
					var document = $(this).parent('div.fileicons').parent('div.document');
					var fileid = $('input[name="fileid"]:hidden', $('div.downloadbutton', document)).val();
					
					$.ajax({
						type: 'POST',				
						dataType: 'html',
						timeout: 8000,
						cache: true,
						data: 'attachments=fileTemplate' + '&fileid=' + fileid + '&' + getParam('token') + '=1',
						beforeSend: function() {
							loading();
						},
						success: function(msg) {
							
							buttonRoutine('edit');
							
							attachments.html(msg);
							
							buttonsUI($('button', attachments));
							
							var form = $('form[name="fileform"]', attachments);
							
							$('input[name="filetitle"]', form).focus();
							
							form.submit(function(e) {
								
								e.preventDefault();
								
								var emptylang = lang('EMPTY_VALUE');
								
								var filetitle = $('input[name="filetitle"]', $(this));
								if (filetitle.val() == '') {
									alert(emptylang);
									filetitle.focus();
									return false;
								}
								
								var file = $('select[name="file"]', $(this));
								if (file.val() == '') {
									alert(emptylang);
									file.focus();
									return false;
								}
								
								$.ajax({
									type: 'POST',				
									dataType: 'html',
									timeout: 8000,
									cache: true,
									data: $(this).serialize(),
									beforeSend: function() {
										loading();
									},
									success: function(msg) {
										if (msg) {
											alert(msg);
											return false;
										} else {
											successInfo(lang('EDITED'));
										}
									},
									error: function() {
										alert(lang('ERROR'));
										return false;
									},
									complete: function() {
										loading();
									}
								});
								
							});
							
						},
						error: function() {
							alert(lang('ERROR'));
							return false;
						},
						complete: function() {
							loading();
						}
					});
				});
			};
			
			/**
			 * Activate file information icon
			 * 
			 * @return void
			 */
			function fileinformationicon() {
				if (!attachments.length) return false;
				
				var isDialogOpen = false;
				
				$('div.document div.fileicons div.fileiconinfo', attachments).click(function() {
					
					if (isDialogOpen) return true; // prevent double dialog calling
					
					var fileinfo = $('div.fileinformation', $(this).parent('div.fileicons').parent('div.document')).clone();
					
					fileinfo.dialog({
						title: lang('FILEINFORMATION'),
						height: 150,
						width: 350,
						position: { at : 'center', my : 'center'},
						draggable: false,
						resizable: false,
						stack: true,
						buttons: {
							'close' : function() {
								$(this).dialog('close');
							}
						},
						open: function() {
							
							attachments.dialog('disable');
							theme($(this));
							
							isDialogOpen = true;
							$('button:first', $(this).next('div.ui-dialog-buttonpane')).focus();
							
							setButtonTitles($(this));
							
						},
						close: function() {
							attachments.dialog('enable');
							isDialogOpen = false;
							removeDialog($(this));
						}
					});
				});
			};
			
			/**
			 * Loading wrapper
			 * 
			 * @param container
			 * @return void
			 */
			function loading() {
				var buttonpanel = $('.ui-dialog-buttonpane', attachments.dialog('widget'));
				
				if ($('span.attachments_loading', buttonpanel).length) {
					$('span.attachments_loading', buttonpanel).remove();
					$('button', buttonpanel).button('enable');
				} else {
					$('button', buttonpanel).button('disable');
					buttonpanel.prepend('<span class="attachments_loading"></span>');
				}
			};
			
			/**
			 * Add to buttons the UI theme
			 * 
			 * @param elements
			 * @return void
			 */
			function buttonsUI(elements) {
				elements.each(function() {
					$(this).button();
				});
			};
			
			/**
			 * Wrapper dialog with related theme
			 * 
			 * @param container
			 * @return void
		 	 */
			function theme(container) {	
				var wrapper = $('<div />', {
					'class' : getParam('uitheme')
				});
				if (typeof(container) == 'undefined') container = attachments;
				container.dialog('widget').wrap(wrapper);
			};
			
			/**
			 * Destroy dialog
			 * 
			 * @param container
			 * @return void
			 */
			function removeDialog(container) {
				var dialog = container.closest('div.' + getParam('uitheme') + '');
				// remove dialog
				dialog.remove();
				// remove container
				container.remove();
			};
			
			/**
			 * Manage button displaying based on layout
			 * 
			 * @param layout
			 * @return void
			 */
			function buttonRoutine(layout) {
				var buttonpane = $('.ui-dialog-buttonpane', attachments.dialog('widget'));
				$('button', buttonpane).each(function() {
					if ($(this).hasClass('ui-state-focus') || $(this).hasClass('ui-state-hover')) {
						$(this).removeClass('ui-state-focus').removeClass('ui-state-hover');
					}
				});
				
				switch (layout) {
					case 'addnew':
					case 'edit':
					case 'state':
					case 'filemgmt':
						$('button.back', buttonpane).show();
						$('button.addnew, button.files', buttonpane).hide();
						break;
					case 'files':
						$('button.files', buttonpane).show();
						$('button.back, button.addnew', buttonpane).hide();
						break;
					default:
						$('button.back', buttonpane).hide();
						$('button.addnew, button.files', buttonpane).show();
						break;
				}
			};
			
			/**
			 * Set button titles
			 * 
			 * @param container
			 * @return void
			 */
			function setButtonTitles(container) {
				
				if (typeof(container) == 'undefined') container = attachments;
				
				var buttons = $('button', $('.ui-dialog-buttonpane', container.closest('.ui-dialog')));
				buttons.each(function() {
					
					switch($(this).button('option', 'label')) {
						case 'close':
							$(this).button('option', 'label', lang('CLOSE')).attr('title', lang('CLOSE'));
							break;
						case 'addnew':
							if (getParam('addnewauth') != '1') {
								$(this).remove();
								return;
							}
							$(this).addClass('addnew');
							$(this).button('option', 'label', lang('ADDNEW')).attr('title', lang('ADDNEW'));
							break;
						case 'files':
							if (getParam('filemanagerauth') != '1') {
								$(this).remove();
								return;
							}
							$(this).addClass('files');
							$(this).button('option', 'label', lang('FILES')).attr('title', lang('FILES'));
							break;
						case 'back':
							if (getParam('addnewauth') != '1' && getParam('filemanagerauth') != '-1') {
								$(this).remove();
								return;
							}
							$(this).addClass('back');
							$(this).button('option', 'label', lang('BACK')).attr('title', lang('BACK'));
							break;
						default:
							break;
					}
				});
			};
			
			/**
			 * Success info message
			 * 
			 * @param msg
			 * @return void
			 */
			function successInfo(msg) {
				attachments.html($('<div><p><span class="state-message ui-icon ui-icon-check"></span>' + msg + '</p></div>'));
				buttonRoutine('state');
			};
			
			/**
			 * Wrong info message
			 * 
			 * @param msg
			 * @return void
			 */
			function wrongInfo(msg) {
				attachments.html($('<iv><p><span class="state-messgae ui-icon ui-icon-alert"></span>' + msg + '</p></div>'));
				buttonRoutine('state');
			};
			
			/**
			 * Get language
			 * 
			 * @return string
			 */
			function lang(string, def) {
				var lang = eval('opts.language.CDATTACHMENTS_' + string);
				if (typeof(lang) === 'string') return lang;
				return def || '';
			};
			
			/**
			 * Get Param
			 * 
			 * @return mixed
			 */
			function getParam(param, def) {
				var output = eval('opts.settings.' + param);
				if (typeof(output) !== 'undefined') return output;
				return def || null;
			};
			
			/**
			 * PHP JS wrapper for "basename()"
			 * 
			 * @param path
			 * @param suffix
			 * @return string
			 */
			function basename (path, suffix) {
				var b=path.replace(/^.*[\/\\]/g,'');if(typeof(suffix)=='string'&&b.substr(b.length-suffix.length)==suffix){b=b.substr(0,b.length-suffix.length)}return b;
			};
			
		};
		
	})(jQuery);
} else {
	alert('Core Design Attachments plugin:\n- Missing jQuery JS!');
}