<?php
/**
 * Core Design Attachments plugin for Joomla! 1.5
 * @author		Daniel Rataj, <info@greatjoomla.com>
 * @package		Joomla
 * @subpackage	Content
 * @category	Plugin
 * @version		1.0.5
 * @copyright	Copyright (C) 2007 - 2010 Great Joomla!, http://www.greatjoomla.com
 * @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL 3
 * 
 * This file is part of Great Joomla! extension.   
 * This extension is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This extension is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Tastymonials table
 *
 * @package 		Joomla.Framework
 * @subpackage		Table
 * @since			1.0
 */
class JTableCdAttachments extends JTable
{
	/** @var int Primary key */
	var $id = null;
	
	/** @var int */
	var $articleid = null;
	
	/** @var int */
	var $userid = null;
	
	/** @var string */
	var $file = null;
	
	/** @var string */
	var $title = null;
	
	/** @var string */
	var $desc = null;
	
	/** @var string */
	var $type = null;
	
	/** @var int */
	var $size = null;
	
	/** @var int */
	var $access = null;
	
	/** @var string */
	var $password = null;
	
	/** @var int */
	var $published = null;
	
	/** @var string */
	var $created = null;
	
	/** @var string */
	var $modified = null;
	
	/** @var int */
	var $modifiedby = null;
	
	/** @var int */
	var $downloaded = null;
	

	/**
	 * A database connector object
	 * 
	 * @param database A database connector object
	 * @return void
	 */
	function __construct( &$db ) {
		parent::__construct( '#__cdattachments', 'id', $db );
		
		// set modified
		$now =& JFactory::getDate();
		$this->set('modified', $now->toMySQL());
		
		$user	=& JFactory::getUser();
		$this->set('modifiedby', $user->id);
	}
	
	/**
	 * Create a database
	 * 
	 * @return boolean
	 */
	function createDb() {
		
		$query = '
		  CREATE TABLE IF NOT EXISTS ' . $this->_tbl . ' (' .
	            '`id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,' .
	        	'`articleid` INT(11) UNSIGNED NOT NULL,' .
	        	'`userid` INT(11) UNSIGNED NOT NULL,' .
	        	'`file` VARCHAR(255) NOT NULL DEFAULT \'\',' .
				'`title` VARCHAR(255) NOT NULL DEFAULT \'\',' .
	        	'`desc` MEDIUMTEXT DEFAULT \'\',' .
	        	'`type` VARCHAR(255) NOT NULL DEFAULT \'\',' .
	        	'`size` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
	            '`access` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
	        	'`password` VARCHAR(255) DEFAULT \'\',' .
	        	'`published` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
	        	'`created` DATETIME DEFAULT \'0000-00-00 00:00:00\',' .
	        	'`modified` DATETIME DEFAULT \'0000-00-00 00:00:00\',' .
				'`modifiedby` INT(11) DEFAULT 0,' .
	        	'`downloaded` INT(11) DEFAULT 0,' .
				' PRIMARY KEY (`id`)' .
	            ') TYPE=MyISAM;';
		
		$this->_db->setQuery($query);
		
		if (!$this->_db->query()) {
			$this->setError($this->_db->stderr());
			return false;
		}
		return true;
	}
	
	/**
	 * Count of messages
	 *
	 * Return count of read and unread messages
	 *
	 * @return int
	 */
	function attachmentsCount($article_id = 0) {
		$articleid = ($article_id ? $article_id : 0);
		$published = 1;
		
		$user	=& JFactory::getUser();
		$access = $user->aid;

		$query = 'SELECT ' . $this->_db->nameQuote('id') .
		' FROM ' . $this->_tbl .
		' WHERE ' . $this->_db->nameQuote('articleid') . ' = ' . $articleid .
		' AND ' . $this->_db->nameQuote('published') . ' = ' . $published .
		' AND ' . $this->_db->nameQuote('access') . ' <= ' . $access . '';

		$this->_db->setQuery($query);
		$rows = count($this->_db->loadObjectList());
		
		return $rows;
	}
	
	/**
	 * Load attachments
	 *
	 * @return object
	 */
	function getItems($orderby = 'modified', $orderdir = 'ASC') {
		$user	=& JFactory::getUser();

		$articleid = (int) JRequest::getInt('articleid', 0, 'post');
		$published = 1;
		$access = $user->aid;
		
		$query = 'SELECT * FROM ' . $this->_tbl .
		' WHERE ' . $this->_db->nameQuote('articleid') . ' = ' . $articleid .
		' AND ' . $this->_db->nameQuote('published') . ' = ' . $published .
		' AND ' . $this->_db->nameQuote('access') . ' <= ' . $access .
		' ORDER BY ' . $this->_tbl . ' . ' . $this->_db->nameQuote($orderby) . ' ' . $orderdir . '';

		$this->_db->setQuery($query);
		
		if ($this->_db->query()) return $this->_db->loadObjectList(); 
		return null;
	}
	
	/**
	 * Hit attachment if download
	 *
	 * @param $id	Attachment ID
	 * @return boolean
	 */
	function hit($id = 0) {
		if (!$this->load($id)) return false;
		$this->set('downloaded', ($this->downloaded + 1));
		if (!$this->store()) return false;
		return true; 
	}
}
?>
