<?php
/**
 * Core Design Attachments plugin for Joomla! 1.7
 * @author		Daniel Rataj, <info@greatjoomla.com>
 * @package		Joomla
 * @subpackage	Content
 * @category	Plugin
 * @version		2.0.2
 * @copyright	Copyright (C) 2007 - 2011 Great Joomla!, http://www.greatjoomla.com
 * @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL 3
 * 
 * This file is part of Great Joomla! extension.   
 * This extension is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This extension is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * Tastymonials table
 *
 * @package 		Joomla.Framework
 * @subpackage		Table
 * @since			1.0
 */
class JTableCdAttachments extends JTable
{
	/** @var int Primary key */
	public $id = null;
	
	/** @public int */
	public $articleid = null;
	
	/** @public int */
	public $userid = null;
	
	/** @public string */
	public $file = null;
	
	/** @public string */
	public $title = null;
	
	/** @public string */
	public $desc = null;
	
	/** @public string */
	public $type = null;
	
	/** @public int */
	public $size = null;
	
	/** @public string */
	public $access = null;
	
	/** @public string */
	public $password = null;
	
	/** @public int */
	public $published = null;
	
	/** @public string */
	public $created = null;
	
	/** @public string */
	public $modified = null;
	
	/** @public int */
	public $modifiedby = null;
	
	/** @public int */
	public $downloaded = null;
	

	/**
	 * A database connector object
	 * 
	 * @param database A database connector object
	 * @return void
	 */
	public function __construct( &$db ) {
		parent::__construct( '#__cdattachments', 'id', $db );
	}
	
	/**
	 * Create a database
	 * 
	 * @return boolean
	 */
	public function createDb() {
		
		$query = '
		  CREATE TABLE IF NOT EXISTS ' . $this->_tbl . ' (' .
	            '`id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,' .
	        	'`articleid` INT(11) UNSIGNED NOT NULL,' .
	        	'`userid` INT(11) UNSIGNED NOT NULL,' .
	        	'`file` VARCHAR(255) NOT NULL DEFAULT \'\',' .
				'`title` VARCHAR(255) NOT NULL DEFAULT \'\',' .
	        	'`desc` MEDIUMTEXT DEFAULT \'\',' .
	        	'`type` VARCHAR(255) NOT NULL DEFAULT \'\',' .
	        	'`size` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
	            '`access` VARCHAR(255) NOT NULL DEFAULT \'\',' .
	        	'`password` VARCHAR(255) DEFAULT \'\',' .
	        	'`published` INT(11) UNSIGNED NOT NULL DEFAULT 0,' .
	        	'`created` DATETIME DEFAULT \'0000-00-00 00:00:00\',' .
	        	'`modified` DATETIME DEFAULT \'0000-00-00 00:00:00\',' .
				'`modifiedby` INT(11) DEFAULT 0,' .
	        	'`downloaded` INT(11) DEFAULT 0,' .
				' PRIMARY KEY (`id`)' .
	            ') ENGINE=MyISAM;';
		
		$this->_db->setQuery($query);
		
		if (!$this->_db->query()) {
			$this->setError($this->_db->stderr());
			return false;
		}
		return true;
	}
	
	/**
	 * Count of messages
	 *
	 * Return count of read and unread messages
	 *
	 * @return int
	 */
	public function attachmentsCount($article_id = 0) {
		$articleid = ($article_id ? $article_id : 0);
		$published = 1;
		
		$query =
		'SELECT ' .
			$this->_db->nameQuote('id') .
			', ' .
			$this->_db->nameQuote('access') .
				' FROM ' . $this->_tbl .
					' WHERE ' . $this->_db->nameQuote('articleid') . ' = ' . $articleid .
					//' AND ' . $this->_db->nameQuote('access') . ' IN( ' . implode(',', JFactory::getUser()->getAuthorisedViewLevels()) . ' )';
					' AND ' . $this->_db->nameQuote('published') . ' = ' . $published;

		$this->_db->setQuery($query);
		
		if (!$this->_db->query()) {
			$this->setError($this->_db->stderr());
			return false;
		}
		
		$rows = array_filter($this->_db->loadObjectList(), array($this, 'cleanArrayFromUnauthorizedLevels'));
		
		return count($rows);
	}
	
	/**
	 * Load attachments
	 *
	 * @return object
	 */
	public function getItems($orderby = 'modified', $orderdir = 'ASC') {
		$user	= JFactory::getUser();

		$articleid = (int) JRequest::getInt('articleid', 0, 'post');
		$published = 1;
		$access = $user->aid;
		
		$query = 'SELECT * FROM ' . $this->_tbl .
		' WHERE ' . $this->_db->nameQuote('articleid') . ' = ' . $articleid .
		' AND ' . $this->_db->nameQuote('published') . ' = ' . $published .
		' ORDER BY ' . $this->_tbl . ' . ' . $this->_db->nameQuote($orderby) . ' ' . $orderdir . '';
		
		$this->_db->setQuery($query);
		
		if (!$this->_db->query()) {
			$this->setError($this->_db->stderr());
			return false;
		}
		
		$rows = array_filter($this->_db->loadObjectList(), array($this, 'cleanArrayFromUnauthorizedLevels'));
		
		return $rows; 
	}
	
	/**
	 * Hit attachment if download
	 *
	 * @param $id	Attachment ID
	 * @return boolean
	 */
	public function hit($id = 0) {
		if (!$this->load($id)) return false;
		$this->set('downloaded', ($this->downloaded + 1));
		if (!$this->store()) return false;
		return true; 
	}
	
	/**
	 * Clean array of objects from unauthorized Joomla! levels
	 * 
	 * @param 	array	$row
	 * @return 	array
	 */
	public function cleanArrayFromUnauthorizedLevels($row = array()) {
		return (array_intersect(JFactory::getUser()->getAuthorisedViewLevels(), explode(',', $row->access)) ? true : false);
	}
	
	public function check() {
		
		// general empty
		if (!$this->get('title')) {
			$this->setError(JText::_('PLG_CONTENT_CDATTACHMENTS_EMPTY_VALUE'));
			return false;
		}
		
		if (!$this->get('access')) {
			$this->setError(JText::_('PLG_CONTENT_CDATTACHMENTS_EMPTY_VALUE'));
			return false;
		}
		
		if (!$this->get('file')) {
			$this->setError(JText::_('PLG_CONTENT_CDATTACHMENTS_EMPTY_VALUE'));
			return false;
		}
		
		if (!$this->get('published')) {
			$this->set('published', 1);
		}
		
		
		// new item
		if ($this->get('id', 0) === 0) {
			if (!$this->get('userid')) {
				$this->set('userid', JFactory::getUser()->get('id' , 0));
			}
			
			if (!$this->get('created')) {
				$this->set('created', JFactory::getDate()->toMySQL());
			}
		} else {
			// edit item
			if (!$this->get('modified')) {
				$this->set('modified', JFactory::getDate()->toMySQL());
			}
			if (!$this->get('modifiedby')) {
				$this->set('modifiedby', JFactory::getUser()->get('id' , 0));
			}
		}
		
		return true;
	}
}
?>
